<?php

namespace Drupal\ai_webform_sentiment;

use Drupal\ai\AiProviderPluginManager;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\ai\Entity\AiClient;

/**
 * Processes the sentiment analysis queue.
 */
class SentimentProcessor {
  use StringTranslationTrait;

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The AI provider manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $aiProviderManager;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a new SentimentProcessor object.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\ai\AiProviderPluginManager $ai_provider_manager
   *   The AI provider manager.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    QueueFactory $queue_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AiProviderPluginManager $ai_provider_manager,
    Connection $database,
    TimeInterface $time
  ) {
    $this->queueFactory = $queue_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->aiProviderManager = $ai_provider_manager;
    $this->database = $database;
    $this->time = $time;
  }

  /**
   * Process sentiment analysis queue items.
   */
  public function processQueue() {
    $queue = $this->queueFactory->get('analyze_sentiment');

    // Debug: Check queue size.
    $queue_size = $queue->numberOfItems();
    \Drupal::logger('ai_webform_sentiment')->notice('Queue size: @size', ['@size' => $queue_size]);

    // Process items from the queue.
    $item_count = 0;
    while ($item = $queue->claimItem()) {
      \Drupal::logger('ai_webform_sentiment')->notice('Processing item @count', ['@count' => $item_count + 1]);
      try {
        $this->processItem($item->data);
        $queue->deleteItem($item);
        $item_count++;
      }
      catch (\Exception $e) {
        // Log the exception and release the item back to the queue.
        \Drupal::logger('ai_webform_sentiment')->error('Error processing sentiment analysis: @message', [
          '@message' => $e->getMessage(),
        ]);
        $queue->releaseItem($item);
      }
    }

    \Drupal::logger('ai_webform_sentiment')->notice('Processed @count items', ['@count' => $item_count]);
    return $item_count;
  }

  /**
   * Process a single queue item.
   *
   * @param array $data
   *   The queue item data.
   */
  public function processItem(array $data) {
    // 1. Load the webform submission.
    \Drupal::logger('ai_webform_sentiment')->notice('load submission', []);
    $submission = $this->entityTypeManager->getStorage('webform_submission')
      ->load($data['submission_id']);

    if (!$submission) {
      \Drupal::logger('ai_webform_sentiment')->error('Could not load webform submission: @id', [
        '@id' => $data['submission_id'],
      ]);
      return;
    }

    // 2. Load the handler configuration to get the prompt.
    \Drupal::logger('ai_webform_sentiment')->notice('load handler', []);
    $webform = $submission->getWebform();
    $handler = $webform->getHandler($data['handler_id']);
    $config = $handler->getConfiguration();
    $prompt = $handler->getConfiguration()['settings']['prompt'];
    $ai_provider_model = $handler->getConfiguration()['settings']['llm_model'] ?? '';

    \Drupal::logger('ai_webform_sentiment')->notice($prompt, []);

    // Replace simple [submission] placeholder if it exists.
    if (strpos($prompt, '[submission]') !== FALSE) {
      $submission_data = '';
      $submission_form_data = $submission->getData();
      foreach ($submission_form_data as $key => $value) {
        if (is_array($value)) {
          $value = implode(', ', $value);
        }
        $submission_data .= $key . ': ' . $value . "\n";
      }
      $prompt = str_replace('[submission]', $submission_data, $prompt);
    }

    // 4. Load an AI provider (using the first available one for now).
    \Drupal::logger('ai_webform_sentiment')->notice('load_ai', []);

    if ($ai_provider_model === '') {
      $default_provider = $this->aiProviderManager->getDefaultProviderForOperationType('chat');
      $ai_provider_model = $default_provider['provider_id'] . '__' . $default_provider['model_id'];
      $ai_model_to_use = $default_provider['model_id'];
    }
    else {
      $parts = explode('__', $ai_provider_model);
      $ai_model_to_use = $parts[1];
    }

    // 6. Call the AI method.
    try {
      $provider = $this->aiProviderManager->loadProviderFromSimpleOption($ai_provider_model);
      $temp = $config['llm_temp'] ?? 0.5;
      $provider->setConfiguration(['temperature' => (float) $temp]);

      $input = new ChatInput([
        new ChatMessage('user', $prompt),
      ]);
      $output = $provider->chat($input, $ai_model_to_use, ['ai_search_block']);
      $response = $output->getNormalized()->getText() . "\n";

      // Extract sentiment from the response.
      $sentiment = $response;

      // Trim to ensure it fits in the database field.
      $sentiment = (int) trim($response);

      // 7. Save the sentiment in the database.
      $current_time = new DrupalDateTime();
      $this->database->insert('ai_webform_sentiment')
        ->fields([
          'webform_id' => $data['webform_id'],
          'submission_id' => $data['submission_id'],
          'datetime' => $current_time->format(DateTimeItemInterface::DATETIME_STORAGE_FORMAT),
          'sentiment' => $sentiment,
        ])
        ->execute();

      \Drupal::logger('ai_webform_sentiment')->notice('Sentiment analysis saved for submission @id: @sentiment', [
        '@id' => $data['submission_id'],
        '@sentiment' => $sentiment,
      ]);
    }
    catch (\Exception $e) {
      \Drupal::logger('ai_webform_sentiment')->error('Error during AI processing: @message', [
        '@message' => $e->getMessage(),
      ]);
      throw $e;
    }
  }
}
